"""Project metadata

Information describing the project.
"""

# The package name, which is also the "UNIX name" for the project.
package = 'ficloud'
project = "ficloud"
project_no_spaces = project.replace(' ', '')
version = '0.1.13'
description = 'Production cloud deployments of fig infrastructure with docker'
authors = ['Alex Rudakov']
authors_string = ', '.join(authors)
emails = ['ribozz@gmail.com']
license = 'Apache License'
copyright = '2013 ' + authors_string
url = 'http://pywizard.com/ficloud/'
